@@ -1,9 +1,11 @@
+# -*- coding: utf-8 -*-
+
 from mutagen._util import DictMixin, cdata, insert_bytes, delete_bytes
-from mutagen._util import decode_terminated, split_escape, dict_match, enum
+from mutagen._util import decode_terminated, dict_match, enum
 from mutagen._util import BitReader, BitReaderError
 from mutagen._compat import text_type, itervalues, iterkeys, iteritems, PY2, \
     cBytesIO, xrange
-from tests import TestCase, add
+from tests import TestCase
 import random
 import mmap
 
@@ -121,8 +123,6 @@ class TDictMixin(TestCase):
         self.failUnlessEqual(self.fdict, self.rdict)
         self.failUnlessEqual(self.rdict, self.fdict)
 
-add(TDictMixin)
-
 
 class Tcdata(TestCase):
 
@@ -260,8 +260,6 @@ class Tcdata(TestCase):
         self.failIf(cdata.test_bit(v, 8))
         self.failIf(cdata.test_bit(v, 13))
 
-add(Tcdata)
-
 
 class FileHandling(TestCase):
     def file(self, contents):
@@ -427,8 +425,6 @@ class FileHandling(TestCase):
             fobj.seek(0)
             self.failUnless(fobj.read() == data)
 
-add(FileHandling)
-
 
 class FileHandlingMockedLock(FileHandling):
 
@@ -441,8 +437,8 @@ class FileHandlingMockedLock(FileHandling):
     def tearDown(self):
         fcntl.lockf = self._orig_lockf
 
-if fcntl:
-    add(FileHandlingMockedLock)
+if not fcntl:
+    del FileHandlingMockedLock
 
 
 class FileHandlingMockedMMapMove(FileHandling):
@@ -464,8 +460,6 @@ class FileHandlingMockedMMapMove(FileHandling):
     def tearDown(self):
         mmap.mmap = self._orig_mmap
 
-add(FileHandlingMockedMMapMove)
-
 
 class FileHandlingMockedMMap(FileHandling):
 
@@ -479,8 +473,6 @@ class FileHandlingMockedMMap(FileHandling):
     def tearDown(self):
         mmap.mmap = self._orig_mmap
 
-add(FileHandlingMockedMMap)
-
 
 class Tdict_match(TestCase):
 
@@ -497,8 +489,6 @@ class Tdict_match(TestCase):
         self.assertEqual(dict_match({"[ab]": 1}, "c"), None)
         self.assertEqual(dict_match({"[ab]": 1}, "[ab]"), None)
 
-add(Tdict_match)
-
 
 class Tenum(TestCase):
 
@@ -516,8 +506,6 @@ class Tenum(TestCase):
         self.assertEqual(str(Foo(42)), "42")
         self.assertEqual(str(Foo(1)), "1")
 
-add(Tenum)
-
 
 class Tdecode_terminated(TestCase):
 
@@ -557,54 +545,6 @@ class Tdecode_terminated(TestCase):
             UnicodeDecodeError, decode_terminated,
             truncated, "utf-8", strict=False)
 
-add(Tdecode_terminated)
-
-
-class Tsplit_escape(TestCase):
-    def test_split_escape(self):
-        inout = [
-            (("", ":"), [""]),
-            ((":", ":"), ["", ""]),
-            ((":", ":", 0), [":"]),
-            ((":b:c:", ":", 0), [":b:c:"]),
-            ((":b:c:", ":", 1), ["", "b:c:"]),
-            ((":b:c:", ":", 2), ["", "b", "c:"]),
-            ((":b:c:", ":", 3), ["", "b", "c", ""]),
-            (("a\\:b:c", ":"), ["a:b", "c"]),
-            (("a\\\\:b:c", ":"), ["a\\", "b", "c"]),
-            (("a\\\\\\:b:c\\:", ":"), ["a\\:b", "c:"]),
-            (("\\", ":"), [""]),
-            (("\\\\", ":"), ["\\"]),
-            (("\\\\a\\b", ":"), ["\\a\\b"]),
-        ]
-
-        for inargs, out in inout:
-            self.assertEqual(split_escape(*inargs), out)
-
-    def test_types(self):
-        parts = split_escape(b"\xff:\xff", b":")
-        self.assertEqual(parts, [b"\xff", b"\xff"])
-        self.assertTrue(isinstance(parts[0], bytes))
-
-        parts = split_escape(b"", b":")
-        self.assertEqual(parts, [b""])
-        self.assertTrue(isinstance(parts[0], bytes))
-
-        parts = split_escape(u"a:b", u":")
-        self.assertEqual(parts, [u"a", u"b"])
-        self.assertTrue(all(isinstance(p, text_type) for p in parts))
-
-        parts = split_escape(u"", u":")
-        self.assertEqual(parts, [u""])
-        self.assertTrue(all(isinstance(p, text_type) for p in parts))
-
-        parts = split_escape(u":", u":")
-        self.assertEqual(parts, [u"", u""])
-        self.assertTrue(all(isinstance(p, text_type) for p in parts))
-
-
-add(Tsplit_escape)
-
 
 class TBitReader(TestCase):
 
@@ -681,5 +621,3 @@ class TBitReader(TestCase):
         self.assertFalse(r.is_aligned())
         r.bits(1)
         self.assertTrue(r.is_aligned())
-
-add(TBitReader)
