@@ -1,22 +1,24 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
 
-from tests import TestCase
+from tests import TestCase, DATA_DIR
 from mutagen._compat import cBytesIO
-from tests import add
 from mutagen.aiff import AIFF, AIFFInfo, delete, IFFFile, IFFChunk
 from mutagen.aiff import error as AIFFError
 from tempfile import mkstemp
 
-class TAIFF(TestCase):
-    silence_1 = os.path.join('tests', 'data', '11k-1ch-2s-silence.aif')
-    silence_2 = os.path.join('tests', 'data', '48k-2ch-s16-silence.aif')
-    silence_3 = os.path.join('tests', 'data', '8k-1ch-1s-silence.aif')
-    silence_4 = os.path.join('tests', 'data', '8k-1ch-3.5s-silence.aif')
-    silence_5 = os.path.join('tests', 'data', '8k-4ch-1s-silence.aif')
 
-    has_tags = os.path.join('tests', 'data', 'with-id3.aif')
-    no_tags  = os.path.join('tests', 'data', '8k-1ch-1s-silence.aif')
+class TAIFF(TestCase):
+    silence_1 = os.path.join(DATA_DIR, '11k-1ch-2s-silence.aif')
+    silence_2 = os.path.join(DATA_DIR, '48k-2ch-s16-silence.aif')
+    silence_3 = os.path.join(DATA_DIR, '8k-1ch-1s-silence.aif')
+    silence_4 = os.path.join(DATA_DIR, '8k-1ch-3.5s-silence.aif')
+    silence_5 = os.path.join(DATA_DIR, '8k-4ch-1s-silence.aif')
+
+    has_tags = os.path.join(DATA_DIR, 'with-id3.aif')
+    no_tags = os.path.join(DATA_DIR, '8k-1ch-1s-silence.aif')
 
     def setUp(self):
         fd, self.filename_1 = mkstemp(suffix='.aif')
@@ -27,7 +29,7 @@ class TAIFF(TestCase):
         os.close(fd)
         shutil.copy(self.no_tags, self.filename_2)
 
-        self.aiff_tmp_id3    = AIFF(self.filename_1)
+        self.aiff_tmp_id3 = AIFF(self.filename_1)
         self.aiff_tmp_no_id3 = AIFF(self.filename_2)
 
         self.aiff_1 = AIFF(self.silence_1)
@@ -73,7 +75,7 @@ class TAIFF(TestCase):
 
     def test_notaiff(self):
         self.failUnlessRaises(
-            AIFFError, AIFF, os.path.join('tests', 'data', 'empty.ofr'))
+            AIFFError, AIFF, os.path.join(DATA_DIR, 'empty.ofr'))
 
     def test_pprint(self):
         self.failUnless(self.aiff_1.pprint())
@@ -118,7 +120,7 @@ class TAIFF(TestCase):
         self.failUnlessEqual(new["TIT2"], ["AIFF title"])
 
     def test_save_tags(self):
-        from mutagen._id3frames import TIT1
+        from mutagen.id3 import TIT1
         tags = self.aiff_tmp_id3.tags
         tags.add(TIT1(encoding=3, text="foobar"))
         tags.save()
@@ -127,23 +129,30 @@ class TAIFF(TestCase):
         self.failUnlessEqual(new["TIT1"], ["foobar"])
 
     def test_save_with_ID3_chunk(self):
-        from mutagen._id3frames import TIT1
+        from mutagen.id3 import TIT1
         self.aiff_tmp_id3["TIT1"] = TIT1(encoding=3, text="foobar")
         self.aiff_tmp_id3.save()
         self.failUnless(AIFF(self.filename_1)["TIT1"] == "foobar")
         self.failUnless(self.aiff_tmp_id3["TIT2"] == "AIFF title")
 
     def test_save_without_ID3_chunk(self):
-        from mutagen._id3frames import TIT1
+        from mutagen.id3 import TIT1
         self.aiff_tmp_no_id3["TIT1"] = TIT1(encoding=3, text="foobar")
         self.aiff_tmp_no_id3.save()
         self.failUnless(AIFF(self.filename_2)["TIT1"] == "foobar")
 
+    def test_corrupt_tag(self):
+        with open(self.filename_1, "r+b") as h:
+            chunk = IFFFile(h)[u'ID3']
+            h.seek(chunk.data_offset)
+            h.seek(4, 1)
+            h.write(b"\xff\xff")
+        self.assertRaises(AIFFError, AIFF, self.filename_1)
+
     def tearDown(self):
         os.unlink(self.filename_1)
         os.unlink(self.filename_2)
 
-add(TAIFF)
 
 class TAIFFInfo(TestCase):
 
@@ -151,33 +160,32 @@ class TAIFFInfo(TestCase):
         fileobj = cBytesIO(b"")
         self.failUnlessRaises(IOError, AIFFInfo, fileobj)
 
-add(TAIFFInfo)
 
 class TIFFFile(TestCase):
-    has_tags = os.path.join('tests', 'data', 'with-id3.aif')
-    no_tags  = os.path.join('tests', 'data', '8k-1ch-1s-silence.aif')
+    has_tags = os.path.join(DATA_DIR, 'with-id3.aif')
+    no_tags = os.path.join(DATA_DIR, '8k-1ch-1s-silence.aif')
 
     def setUp(self):
         self.file_1 = open(self.has_tags, 'rb')
-        self.iff_1  = IFFFile(self.file_1)
+        self.iff_1 = IFFFile(self.file_1)
         self.file_2 = open(self.no_tags, 'rb')
-        self.iff_2  = IFFFile(self.file_2)
+        self.iff_2 = IFFFile(self.file_2)
 
         fd_1, tmp_1_name = mkstemp(suffix='.aif')
         shutil.copy(self.has_tags, tmp_1_name)
         self.file_1_tmp = open(tmp_1_name, 'rb+')
-        self.iff_1_tmp  = IFFFile(self.file_1_tmp)
+        self.iff_1_tmp = IFFFile(self.file_1_tmp)
 
         fd_2, tmp_2_name = mkstemp(suffix='.aif')
         shutil.copy(self.no_tags, tmp_2_name)
         self.file_2_tmp = open(tmp_2_name, 'rb+')
-        self.iff_2_tmp  = IFFFile(self.file_2_tmp)
+        self.iff_2_tmp = IFFFile(self.file_2_tmp)
 
     def test_has_chunks(self):
         self.failUnless('FORM' in self.iff_1)
         self.failUnless('COMM' in self.iff_1)
         self.failUnless('SSND' in self.iff_1)
-        self.failUnless('ID3'  in self.iff_1)
+        self.failUnless('ID3' in self.iff_1)
 
         self.failUnless('FORM' in self.iff_2)
         self.failUnless('COMM' in self.iff_2)
@@ -187,7 +195,7 @@ class TIFFFile(TestCase):
         self.failUnless(isinstance(self.iff_1['FORM'], IFFChunk))
         self.failUnless(isinstance(self.iff_1['COMM'], IFFChunk))
         self.failUnless(isinstance(self.iff_1['SSND'], IFFChunk))
-        self.failUnless(isinstance(self.iff_1['ID3'],  IFFChunk))
+        self.failUnless(isinstance(self.iff_1['ID3'], IFFChunk))
 
     def test_chunk_size(self):
         self.failUnlessEqual(self.iff_1['FORM'].size, 17096)
@@ -234,5 +242,3 @@ class TIFFFile(TestCase):
         self.file_2.close()
         self.file_1_tmp.close()
         self.file_2_tmp.close()
-
-add(TIFFFile)
