@@ -1,8 +1,10 @@
+# -*- coding: utf-8 -*-
+
 import shutil
 import os
-from os.path import devnull
+import subprocess
 
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR
 
 from mutagen.id3 import ID3, TIT2, ID3NoHeaderError
 from mutagen.flac import to_int_be, Padding, VCFLACDict, MetadataBlock, error
@@ -11,6 +13,12 @@ from mutagen._compat import PY3
 from tests.test__vorbis import TVCommentDict, VComment
 
 
+def call_flac(*args):
+    with open(os.devnull, 'wb') as null:
+        return subprocess.call(
+            ["flac"] + list(args), stdout=null, stderr=subprocess.STDOUT)
+
+
 class Tto_int_be(TestCase):
 
     def test_empty(self):
@@ -27,7 +35,6 @@ class Tto_int_be(TestCase):
 
     def test_long(self):
         self.failUnlessEqual(to_int_be(b"\x01\x00\x00\x00\x00"), 2 ** 32)
-add(Tto_int_be)
 
 
 class TVCFLACDict(TVCommentDict):
@@ -36,7 +43,6 @@ class TVCFLACDict(TVCommentDict):
 
     def test_roundtrip_vc(self):
         self.failUnlessEqual(self.c, VComment(self.c.write() + b"\x01"))
-add(TVCFLACDict)
 
 
 class TMetadataBlock(TestCase):
@@ -68,7 +74,6 @@ class TMetadataBlock(TestCase):
         length2 = len(MetadataBlock.writeblocks(blocks))
         self.failUnlessEqual(length1, length2)
         self.failUnlessEqual(len(blocks), 2)
-add(TMetadataBlock)
 
 
 class TStreamInfo(TestCase):
@@ -118,11 +123,10 @@ class TStreamInfo(TestCase):
 
     def test_roundtrip(self):
         self.failUnlessEqual(StreamInfo(self.i.write()), self.i)
-add(TStreamInfo)
 
 
 class TSeekTable(TestCase):
-    SAMPLE = os.path.join("tests", "data", "silence-44-s.flac")
+    SAMPLE = os.path.join(DATA_DIR, "silence-44-s.flac")
 
     def setUp(self):
         self.flac = FLAC(self.SAMPLE)
@@ -148,11 +152,10 @@ class TSeekTable(TestCase):
 
     def test_roundtrip(self):
         self.failUnlessEqual(SeekTable(self.st.write()), self.st)
-add(TSeekTable)
 
 
 class TCueSheet(TestCase):
-    SAMPLE = os.path.join("tests", "data", "silence-44-s.flac")
+    SAMPLE = os.path.join(DATA_DIR, "silence-44-s.flac")
 
     def setUp(self):
         self.flac = FLAC(self.SAMPLE)
@@ -205,11 +208,10 @@ class TCueSheet(TestCase):
 
     def test_roundtrip(self):
         self.failUnlessEqual(CueSheet(self.cs.write()), self.cs)
-add(TCueSheet)
 
 
 class TPicture(TestCase):
-    SAMPLE = os.path.join("tests", "data", "silence-44-s.flac")
+    SAMPLE = os.path.join(DATA_DIR, "silence-44-s.flac")
 
     def setUp(self):
         self.flac = FLAC(self.SAMPLE)
@@ -239,7 +241,6 @@ class TPicture(TestCase):
 
     def test_roundtrip(self):
         self.failUnlessEqual(Picture(self.p.write()), self.p)
-add(TPicture)
 
 
 class TPadding(TestCase):
@@ -262,11 +263,10 @@ class TPadding(TestCase):
     def test_change(self):
         self.b.length = 20
         self.failUnlessEqual(self.b.write(), b"\x00" * 20)
-add(TPadding)
 
 
 class TFLAC(TestCase):
-    SAMPLE = os.path.join("tests", "data", "silence-44-s.flac")
+    SAMPLE = os.path.join(DATA_DIR, "silence-44-s.flac")
     NEW = SAMPLE + ".new"
 
     def setUp(self):
@@ -366,14 +366,14 @@ class TFLAC(TestCase):
         self.failUnlessEqual(f["faketag"], ["foo"])
 
     def test_add_vc(self):
-        f = FLAC(os.path.join("tests", "data", "no-tags.flac"))
+        f = FLAC(os.path.join(DATA_DIR, "no-tags.flac"))
         self.failIf(f.tags)
         f.add_tags()
         self.failUnless(f.tags == [])
         self.failUnlessRaises(ValueError, f.add_tags)
 
     def test_add_vc_implicit(self):
-        f = FLAC(os.path.join("tests", "data", "no-tags.flac"))
+        f = FLAC(os.path.join(DATA_DIR, "no-tags.flac"))
         self.failIf(f.tags)
         f["foo"] = "bar"
         self.failUnless(f.tags == [("foo", "bar")])
@@ -382,7 +382,7 @@ class TFLAC(TestCase):
     def test_ooming_vc_header(self):
         # issue 112: Malformed FLAC Vorbis header causes out of memory error
         # http://code.google.com/p/mutagen/issues/detail?id=112
-        self.assertRaises(IOError, FLAC, os.path.join('tests', 'data',
+        self.assertRaises(IOError, FLAC, os.path.join(DATA_DIR,
                                                       'ooming-header.flac'))
 
     def test_with_real_flac(self):
@@ -390,9 +390,7 @@ class TFLAC(TestCase):
             return
         self.flac["faketag"] = "foobar" * 1000
         self.flac.save()
-        badval = os.system("tools/notarealprogram 2> %s" % devnull)
-        value = os.system("flac -t %s 2> %s" % (self.flac.filename, devnull))
-        self.failIf(value and value != badval)
+        self.failIf(call_flac("-t", self.flac.filename) != 0)
 
     def test_save_unknown_block(self):
         block = MetadataBlock(b"test block data")
@@ -419,11 +417,11 @@ class TFLAC(TestCase):
 
     def test_load_invalid_flac(self):
         self.failUnlessRaises(
-            IOError, FLAC, os.path.join("tests", "data", "xing.mp3"))
+            IOError, FLAC, os.path.join(DATA_DIR, "xing.mp3"))
 
     def test_save_invalid_flac(self):
         self.failUnlessRaises(
-            IOError, self.flac.save, os.path.join("tests", "data", "xing.mp3"))
+            IOError, self.flac.save, os.path.join(DATA_DIR, "xing.mp3"))
 
     def test_pprint(self):
         self.failUnless(self.flac.pprint())
@@ -486,14 +484,13 @@ class TFLAC(TestCase):
         self.failUnless("audio/x-flac" in self.flac.mime)
 
     def test_variable_block_size(self):
-        FLAC(os.path.join("tests", "data", "variable-block.flac"))
+        FLAC(os.path.join(DATA_DIR, "variable-block.flac"))
 
     def test_load_flac_with_application_block(self):
-        FLAC(os.path.join("tests", "data", "flac_application.flac"))
+        FLAC(os.path.join(DATA_DIR, "flac_application.flac"))
 
     def tearDown(self):
         os.unlink(self.NEW)
-add(TFLAC)
 
 
 class TFLACFile(TestCase):
@@ -501,18 +498,15 @@ class TFLACFile(TestCase):
     def test_open_nonexistant(self):
         """mutagen 1.2 raises UnboundLocalError, then it tries to open
         non-existant FLAC files"""
-        filename = os.path.join("tests", "data", "doesntexist.flac")
+        filename = os.path.join(DATA_DIR, "doesntexist.flac")
         self.assertRaises(IOError, FLAC, filename)
 
-add(TFLACFile)
-
 
 class TFLACBadBlockSize(TestCase):
-    TOO_SHORT = os.path.join("tests", "data", "52-too-short-block-size.flac")
-    TOO_SHORT_2 = os.path.join("tests", "data",
-                               "106-short-picture-block-size.flac")
-    OVERWRITTEN = os.path.join("tests", "data", "52-overwritten-metadata.flac")
-    INVAL_INFO = os.path.join("tests", "data", "106-invalid-streaminfo.flac")
+    TOO_SHORT = os.path.join(DATA_DIR, "52-too-short-block-size.flac")
+    TOO_SHORT_2 = os.path.join(DATA_DIR, "106-short-picture-block-size.flac")
+    OVERWRITTEN = os.path.join(DATA_DIR, "52-overwritten-metadata.flac")
+    INVAL_INFO = os.path.join(DATA_DIR, "106-invalid-streaminfo.flac")
 
     def test_too_short_read(self):
         flac = FLAC(self.TOO_SHORT)
@@ -528,11 +522,10 @@ class TFLACBadBlockSize(TestCase):
 
     def test_inval_streaminfo(self):
         self.assertRaises(error, FLAC, self.INVAL_INFO)
-add(TFLACBadBlockSize)
 
 
 class TFLACBadBlockSizeWrite(TestCase):
-    TOO_SHORT = os.path.join("tests", "data", "52-too-short-block-size.flac")
+    TOO_SHORT = os.path.join(DATA_DIR, "52-too-short-block-size.flac")
     NEW = TOO_SHORT + ".new"
 
     def setUp(self):
@@ -550,8 +543,6 @@ class TFLACBadBlockSizeWrite(TestCase):
     def tearDown(self):
         os.unlink(self.NEW)
 
-add(TFLACBadBlockSizeWrite)
-
 
 class CVE20074619(TestCase):
 
@@ -563,8 +554,11 @@ class CVE20074619(TestCase):
         # "Editing any Metadata Block Size value to a large value such
         # as 0xFFFFFFFF may result in a heap based overflow in the
         # decoding software."
-        filename = os.path.join("tests", "data", "CVE-2007-4619-1.flac")
+        filename = os.path.join(DATA_DIR, "CVE-2007-4619-1.flac")
         self.failUnlessRaises(IOError, FLAC, filename)
+        # work around https://bitbucket.org/pypy/pypy/issue/1988
+        import gc
+        gc.collect()
 
     def test_2(self):
         # "The second vulnerability lies within the parsing of any
@@ -572,8 +566,11 @@ class CVE20074619(TestCase):
         # an overly large size, such as 0xFFFFFFF, could also result
         # in another heap-based overflow allowing arbitrary code to
         # execute in the content of the decoding program."
-        filename = os.path.join("tests", "data", "CVE-2007-4619-2.flac")
+        filename = os.path.join(DATA_DIR, "CVE-2007-4619-2.flac")
         self.failUnlessRaises(IOError, FLAC, filename)
+        # work around https://bitbucket.org/pypy/pypy/issue/1988
+        import gc
+        gc.collect()
 
     # "By inserting an overly long VORBIS Comment data string along
     # with an large VORBIS Comment data string size value (such as
@@ -604,11 +601,11 @@ class CVE20074619(TestCase):
     # impossible to store in a FLAC file.
 
     def test_12_read(self):
-        filename = os.path.join("tests", "data", "CVE-2007-4619-12.flac")
+        filename = os.path.join(DATA_DIR, "CVE-2007-4619-12.flac")
         self.failUnlessRaises(IOError, FLAC, filename)
 
     def test_12_write_too_big(self):
-        filename = os.path.join("tests", "data", "silence-44-s.flac")
+        filename = os.path.join(DATA_DIR, "silence-44-s.flac")
         f = FLAC(filename)
         # This size is too big to be an integer.
         f.metadata_blocks[-1].length = 0xFFFFFFFFFFFFFFFF
@@ -616,7 +613,7 @@ class CVE20074619(TestCase):
 
     def test_12_write_too_big_for_flac(self):
         from mutagen.flac import MetadataBlock
-        filename = os.path.join("tests", "data", "silence-44-s.flac")
+        filename = os.path.join(DATA_DIR, "silence-44-s.flac")
         f = FLAC(filename)
         # This size is too big to be in a FLAC block but is overwise fine.
         f.metadata_blocks[-1].length = 0x1FFFFFF
@@ -626,11 +623,10 @@ class CVE20074619(TestCase):
     # Vulnerability 13 and 14 are specific to libFLAC and C/C++ memory
     # management schemes.
 
-add(CVE20074619)
-
-NOTFOUND = os.system("tools/notarealprogram 2> %s" % devnull)
 
 have_flac = True
-if os.system("flac 2> %s > %s" % (devnull, devnull)) == NOTFOUND:
+try:
+    call_flac()
+except OSError:
     have_flac = False
     print("WARNING: Skipping FLAC reference tests.")
