@@ -1,21 +1,24 @@
+# -*- coding: utf-8 -*-
+
 import os
 from os.path import join
 import shutil
-from tests import TestCase
-from tests import add
+from tests import TestCase, DATA_DIR
 from mutagen import id3
 from mutagen.apev2 import APEv2
 from mutagen.id3 import ID3, COMR, Frames, Frames_2_2, ID3Warning, \
-    ID3JunkFrameError
+    ID3JunkFrameError, ID3Header, ID3UnsupportedVersionError, _fullread
+from mutagen.id3._util import BitPaddedInt, error as ID3Error
 from mutagen._compat import cBytesIO, PY2, iteritems, integer_types
 import warnings
+from tempfile import mkstemp
 warnings.simplefilter('error', ID3Warning)
 
-_22 = ID3()
+_22 = ID3Header()
 _22.version = (2, 2, 0)
-_23 = ID3()
+_23 = ID3Header()
 _23.version = (2, 3, 0)
-_24 = ID3()
+_24 = ID3Header()
 _24.version = (2, 4, 0)
 
 
@@ -75,116 +78,102 @@ class ID3GetSetDel(TestCase):
 
 class ID3Loading(TestCase):
 
-    empty = join('tests', 'data', 'emptyfile.mp3')
-    silence = join('tests', 'data', 'silence-44-s.mp3')
-    unsynch = join('tests', 'data', 'id3v23_unsynch.id3')
+    empty = join(DATA_DIR, 'emptyfile.mp3')
+    silence = join(DATA_DIR, 'silence-44-s.mp3')
+    unsynch = join(DATA_DIR, 'id3v23_unsynch.id3')
 
     def test_empty_file(self):
         name = self.empty
-        self.assertRaises(ValueError, ID3, filename=name)
+        self.assertRaises(ID3Error, ID3, filename=name)
 
     def test_nonexistent_file(self):
-        name = join('tests', 'data', 'does', 'not', 'exist')
+        name = join(DATA_DIR, 'does', 'not', 'exist')
         self.assertRaises(EnvironmentError, ID3, name)
 
     def test_header_empty(self):
-        id3 = ID3()
-        id3._fileobj = open(self.empty, 'rb')
-        self.assertRaises(EOFError, id3._load_header)
+        fileobj = open(self.empty, 'rb')
+        self.assertRaises(ID3Error, ID3Header, fileobj)
 
     def test_header_silence(self):
-        id3 = ID3()
-        id3._fileobj = open(self.silence, 'rb')
-        id3._load_header()
-        self.assertEquals(id3.version, (2, 3, 0))
-        self.assertEquals(id3.size, 1314)
+        fileobj = open(self.silence, 'rb')
+        header = ID3Header(fileobj)
+        self.assertEquals(header.version, (2, 3, 0))
+        self.assertEquals(header.size, 1314)
 
     def test_header_2_4_invalid_flags(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(b'ID3\x04\x00\x1f\x00\x00\x00\x00')
-        self.assertRaises(ValueError, id3._load_header)
+        fileobj = cBytesIO(b'ID3\x04\x00\x1f\x00\x00\x00\x00')
+        self.assertRaises(ID3Error, ID3Header, fileobj)
 
     def test_header_2_4_unsynch_size(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(b'ID3\x04\x00\x10\x00\x00\x00\xFF')
-        self.assertRaises(ValueError, id3._load_header)
+        fileobj = cBytesIO(b'ID3\x04\x00\x10\x00\x00\x00\xFF')
+        self.assertRaises(ID3Error, ID3Header, fileobj)
 
     def test_header_2_4_allow_footer(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(b'ID3\x04\x00\x10\x00\x00\x00\x00')
-        id3._load_header()
+        fileobj = cBytesIO(b'ID3\x04\x00\x10\x00\x00\x00\x00')
+        self.assertTrue(ID3Header(fileobj).f_footer)
 
     def test_header_2_3_invalid_flags(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(b'ID3\x03\x00\x1f\x00\x00\x00\x00')
-        self.assertRaises(ValueError, id3._load_header)
-        id3._fileobj = cBytesIO(b'ID3\x03\x00\x0f\x00\x00\x00\x00')
-        self.assertRaises(ValueError, id3._load_header)
+        fileobj = cBytesIO(b'ID3\x03\x00\x1f\x00\x00\x00\x00')
+        self.assertRaises(ID3Error, ID3Header, fileobj)
+
+        fileobj = cBytesIO(b'ID3\x03\x00\x0f\x00\x00\x00\x00')
+        self.assertRaises(ID3Error, ID3Header, fileobj)
 
     def test_header_2_2(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(b'ID3\x02\x00\x00\x00\x00\x00\x00')
-        id3._load_header()
-        self.assertEquals(id3.version, (2, 2, 0))
+        fileobj = cBytesIO(b'ID3\x02\x00\x00\x00\x00\x00\x00')
+        header = ID3Header(fileobj)
+        self.assertEquals(header.version, (2, 2, 0))
 
     def test_header_2_1(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(b'ID3\x01\x00\x00\x00\x00\x00\x00')
-        self.assertRaises(NotImplementedError, id3._load_header)
+        fileobj = cBytesIO(b'ID3\x01\x00\x00\x00\x00\x00\x00')
+        self.assertRaises(ID3UnsupportedVersionError, ID3Header, fileobj)
 
     def test_header_too_small(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(b'ID3\x01\x00\x00\x00\x00\x00')
-        self.assertRaises(EOFError, id3._load_header)
+        fileobj = cBytesIO(b'ID3\x01\x00\x00\x00\x00\x00')
+        self.assertRaises(ID3Error, ID3Header, fileobj)
 
     def test_header_2_4_extended(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(
+        fileobj = cBytesIO(
             b'ID3\x04\x00\x40\x00\x00\x00\x00\x00\x00\x00\x05\x5a')
-        id3._load_header()
-        self.assertEquals(id3._ID3__extsize, 1)
-        self.assertEquals(id3._ID3__extdata, b'\x5a')
+        header = ID3Header(fileobj)
+        self.assertEquals(header._extdata, b'\x5a')
 
     def test_header_2_4_extended_unsynch_size(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(
+        fileobj = cBytesIO(
             b'ID3\x04\x00\x40\x00\x00\x00\x00\x00\x00\x00\xFF\x5a')
-        self.assertRaises(ValueError, id3._load_header)
+        self.assertRaises(ID3Error, ID3Header, fileobj)
 
     def test_header_2_4_extended_but_not(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(
+        fileobj = cBytesIO(
             b'ID3\x04\x00\x40\x00\x00\x00\x00TIT1\x00\x00\x00\x01a')
-        id3._load_header()
-        self.assertEquals(id3._ID3__extsize, 0)
-        self.assertEquals(id3._ID3__extdata, b'')
+        header = ID3Header(fileobj)
+        self.assertEquals(header._extdata, b'')
 
     def test_header_2_4_extended_but_not_but_not_tag(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(
-            b'ID3\x04\x00\x40\x00\x00\x00\x00TIT9')
-        self.failUnlessRaises(EOFError, id3._load_header)
+        fileobj = cBytesIO(b'ID3\x04\x00\x40\x00\x00\x00\x00TIT9')
+        self.failUnlessRaises(ID3Error, ID3Header, fileobj)
 
     def test_header_2_3_extended(self):
-        id3 = ID3()
-        id3._fileobj = cBytesIO(
+        fileobj = cBytesIO(
             b'ID3\x03\x00\x40\x00\x00\x00\x00\x00\x00\x00\x06'
             b'\x00\x00\x56\x78\x9a\xbc')
-        id3._load_header()
-        self.assertEquals(id3._ID3__extsize, 6)
-        self.assertEquals(id3._ID3__extdata, b'\x00\x00\x56\x78\x9a\xbc')
+        header = ID3Header(fileobj)
+        self.assertEquals(header._extdata, b'\x00\x00\x56\x78\x9a\xbc')
 
     def test_unsynch(self):
-        id3 = ID3()
-        id3.version = (2, 4, 0)
-        id3._ID3__flags = 0x80
+        header = ID3Header()
+        header.version = (2, 4, 0)
+        header._flags = 0x80
         badsync = b'\x00\xff\x00ab\x00'
+
         self.assertEquals(
-            id3._ID3__load_framedata(Frames["TPE2"], 0, badsync), [u"\xffab"])
-        id3._ID3__flags = 0x00
-        self.assertEquals(id3._ID3__load_framedata(
-            Frames["TPE2"], 0x02, badsync), [u"\xffab"])
-        tag = id3._ID3__load_framedata(Frames["TPE2"], 0, badsync)
+            Frames["TPE2"]._fromData(header, 0, badsync), [u"\xffab"])
+
+        header._flags = 0x00
+        self.assertEquals(
+            Frames["TPE2"]._fromData(header, 0x02, badsync), [u"\xffab"])
+
+        tag = Frames["TPE2"]._fromData(header, 0, badsync)
         self.assertEquals(tag, [u"\xff", u"ab"])
 
     def test_load_v23_unsynch(self):
@@ -192,10 +181,9 @@ class ID3Loading(TestCase):
         self.assertEquals(id3["TPE1"], ["Nina Simone"])
 
     def test_insane__ID3__fullread(self):
-        id3 = ID3()
-        id3._ID3__filesize = 0
-        self.assertRaises(ValueError, id3._ID3__fullread, -3)
-        self.assertRaises(EOFError, id3._ID3__fullread, 3)
+        fileobj = cBytesIO()
+        self.assertRaises(ValueError, _fullread, fileobj, -3)
+        self.assertRaises(EOFError, _fullread, fileobj, 3)
 
 
 class Issue21(TestCase):
@@ -204,7 +192,7 @@ class Issue21(TestCase):
     # Ensure the extended header is turned off, and the frames are
     # read.
     def setUp(self):
-        self.id3 = ID3(join('tests', 'data', 'issue_21.id3'))
+        self.id3 = ID3(join(DATA_DIR, 'issue_21.id3'))
 
     def test_no_ext(self):
         self.failIf(self.id3.f_extended)
@@ -215,19 +203,24 @@ class Issue21(TestCase):
 
     def test_tit2_value(self):
         self.failUnlessEqual(self.id3["TIT2"].text, [u"Punk To Funk"])
-add(Issue21)
 
 
 class ID3Tags(TestCase):
 
     def setUp(self):
-        self.silence = join('tests', 'data', 'silence-44-s.mp3')
+        self.silence = join(DATA_DIR, 'silence-44-s.mp3')
 
     def test_None(self):
         id3 = ID3(self.silence, known_frames={})
         self.assertEquals(0, len(id3.keys()))
         self.assertEquals(9, len(id3.unknown_frames))
 
+    def test_unknown_reset(self):
+        id3 = ID3(self.silence, known_frames={})
+        self.assertEquals(9, len(id3.unknown_frames))
+        id3.load(self.silence, known_frames={})
+        self.assertEquals(9, len(id3.unknown_frames))
+
     def test_has_docs(self):
         for Kind in (list(Frames.values()) + list(Frames_2_2.values())):
             self.failUnless(Kind.__doc__, "%s has no docstring" % Kind)
@@ -270,50 +263,49 @@ class ID3Tags(TestCase):
 
     def test_badencoding(self):
         self.assertRaises(
-            IndexError, Frames["TPE1"].fromData, _24, 0, b"\x09ab")
+            ID3JunkFrameError, Frames["TPE1"]._fromData, _24, 0, b"\x09ab")
         self.assertRaises(ValueError, Frames["TPE1"], encoding=9, text="ab")
 
     def test_badsync(self):
-        self.assertRaises(
-            ValueError, Frames["TPE1"].fromData, _24, 0x02, b"\x00\xff\xfe")
+        frame = Frames["TPE1"]._fromData(_24, 0x02, b"\x00\xff\xfe")
+        self.assertEqual(frame.text, [u'\xff\xfe'])
 
     def test_noencrypt(self):
         self.assertRaises(
-            NotImplementedError, Frames["TPE1"].fromData, _24, 0x04, b"\x00")
+            NotImplementedError, Frames["TPE1"]._fromData, _24, 0x04, b"\x00")
         self.assertRaises(
-            NotImplementedError, Frames["TPE1"].fromData, _23, 0x40, b"\x00")
+            NotImplementedError, Frames["TPE1"]._fromData, _23, 0x40, b"\x00")
 
     def test_badcompress(self):
         self.assertRaises(
-            ValueError, Frames["TPE1"].fromData, _24, 0x08,
+            ValueError, Frames["TPE1"]._fromData, _24, 0x08,
             b"\x00\x00\x00\x00#")
         self.assertRaises(
-            ValueError, Frames["TPE1"].fromData, _23, 0x80,
+            ValueError, Frames["TPE1"]._fromData, _23, 0x80,
             b"\x00\x00\x00\x00#")
 
     def test_junkframe(self):
-        self.assertRaises(ValueError, Frames["TPE1"].fromData, _24, 0, b"")
+        self.assertRaises(ValueError, Frames["TPE1"]._fromData, _24, 0, b"")
 
     def test_bad_sylt(self):
         self.assertRaises(
-            ID3JunkFrameError, Frames["SYLT"].fromData, _24, 0x0,
+            ID3JunkFrameError, Frames["SYLT"]._fromData, _24, 0x0,
             b"\x00eng\x01description\x00foobar")
         self.assertRaises(
-            ID3JunkFrameError, Frames["SYLT"].fromData, _24, 0x0,
+            ID3JunkFrameError, Frames["SYLT"]._fromData, _24, 0x0,
             b"\x00eng\x01description\x00foobar\x00\xFF\xFF\xFF")
 
     def test_extradata(self):
         from mutagen.id3 import RVRB, RBUF
-        self.assertRaises(ID3Warning, RVRB()._readData, b'L1R1BBFFFFPP#xyz')
-        self.assertRaises(
-            ID3Warning, RBUF()._readData,
-            b'\x00\x01\x00\x01\x00\x00\x00\x00#xyz')
+        self.assertEqual(RVRB()._readData(b'L1R1BBFFFFPP#xyz'), b'#xyz')
+        self.assertEqual(
+            RBUF()._readData(b'\x00\x01\x00\x01\x00\x00\x00\x00#xyz'), b'#xyz')
 
 
 class ID3v1Tags(TestCase):
 
     def setUp(self):
-        self.silence = join('tests', 'data', 'silence-44-s-v1.mp3')
+        self.silence = join(DATA_DIR, 'silence-44-s-v1.mp3')
         self.id3 = ID3(self.silence)
 
     def test_album(self):
@@ -412,10 +404,9 @@ class ID3v1Tags(TestCase):
 
 
 class TestWriteID3v1(TestCase):
-    SILENCE = os.path.join("tests", "data", "silence-44-s.mp3")
+    SILENCE = os.path.join(DATA_DIR, "silence-44-s.mp3")
 
     def setUp(self):
-        from tempfile import mkstemp
         fd, self.filename = mkstemp(suffix='.mp3')
         os.close(fd)
         shutil.copy(self.SILENCE, self.filename)
@@ -452,22 +443,19 @@ class TestWriteID3v1(TestCase):
     def tearDown(self):
         os.unlink(self.filename)
 
-add(TestWriteID3v1)
-
 
 class TestV22Tags(TestCase):
 
     def setUp(self):
-        filename = os.path.join("tests", "data", "id3v22-test.mp3")
+        filename = os.path.join(DATA_DIR, "id3v22-test.mp3")
         self.tags = ID3(filename)
 
     def test_tags(self):
         self.failUnless(self.tags["TRCK"].text == ["3/11"])
         self.failUnless(self.tags["TPE1"].text == ["Anais Mitchell"])
-add(TestV22Tags)
 
 
-def TestReadTags():
+def create_read_tag_tests():
     tests = [
         ['TALB', b'\x00a/b', 'a/b', '', dict(encoding=0)],
         ['TBPM', b'\x00120', '120', 120, dict(encoding=0)],
@@ -874,7 +862,7 @@ def TestReadTags():
             from operator import pos
             id3 = __import__('mutagen.id3', globals(), locals(), [tag])
             TAG = getattr(id3, tag)
-            tag = TAG.fromData(_23, 0, data)
+            tag = TAG._fromData(_23, 0, data)
             self.failUnless(tag.HashKey)
             self.failUnless(tag.pprint())
             self.assertEquals(value, tag)
@@ -902,7 +890,7 @@ def TestReadTags():
             from mutagen.id3 import ID3TimeStamp
             id3 = __import__('mutagen.id3', globals(), locals(), [tag])
             TAG = getattr(id3, tag)
-            tag = TAG.fromData(_23, 0, data)
+            tag = TAG._fromData(_23, 0, data)
             tag2 = eval(repr(tag), {TAG.__name__: TAG,
                         'ID3TimeStamp': ID3TimeStamp})
             self.assertEquals(type(tag), type(tag2))
@@ -923,20 +911,23 @@ def TestReadTags():
         def test_tag_write(self, tag=tag, data=data):
             id3 = __import__('mutagen.id3', globals(), locals(), [tag])
             TAG = getattr(id3, tag)
-            tag = TAG.fromData(_24, 0, data)
+            tag = TAG._fromData(_24, 0, data)
             towrite = tag._writeData()
-            tag2 = TAG.fromData(_24, 0, towrite)
+            tag2 = TAG._fromData(_24, 0, towrite)
             for spec in TAG._framespec:
                 attr = spec.name
                 self.assertEquals(getattr(tag, attr), getattr(tag2, attr))
         write_tests['test_write_%s_%d' % (tag, i)] = test_tag_write
 
     testcase = type('TestReadTags', (TestCase,), load_tests)
-    add(testcase)
+    assert testcase.__name__ not in globals()
+    globals()[testcase.__name__] = testcase
     testcase = type('TestReadReprTags', (TestCase,), repr_tests)
-    add(testcase)
+    assert testcase.__name__ not in globals()
+    globals()[testcase.__name__] = testcase
     testcase = type('TestReadWriteTags', (TestCase,), write_tests)
-    add(testcase)
+    assert testcase.__name__ not in globals()
+    globals()[testcase.__name__] = testcase
 
     from mutagen.id3 import Frames, Frames_2_2
     check = dict.fromkeys(list(Frames.keys()) + list(Frames_2_2.keys()))
@@ -946,10 +937,10 @@ def TestReadTags():
             self.assert_(tag in tested_tags)
         tested_tags['test_' + tag + '_tested'] = check
     testcase = type('TestTestedTags', (TestCase,), tested_tags)
-    add(testcase)
+    assert testcase.__name__ not in globals()
+    globals()[testcase.__name__] = testcase
 
-TestReadTags()
-del TestReadTags
+create_read_tag_tests()
 
 
 class UpdateTo24(TestCase):
@@ -1021,14 +1012,11 @@ class UpdateTo24(TestCase):
         id3.update_to_v24()
         self.assertFalse(id3.getall("TIME"))
 
-add(UpdateTo24)
-
 
 class Issue97_UpgradeUnknown23(TestCase):
-    SILENCE = os.path.join("tests", "data", "97-unknown-23-update.mp3")
+    SILENCE = os.path.join(DATA_DIR, "97-unknown-23-update.mp3")
 
     def setUp(self):
-        from tempfile import mkstemp
         fd, self.filename = mkstemp(suffix='.mp3')
         os.close(fd)
         shutil.copy(self.SILENCE, self.filename)
@@ -1078,28 +1066,28 @@ class Issue97_UpgradeUnknown23(TestCase):
     def tearDown(self):
         os.unlink(self.filename)
 
-add(Issue97_UpgradeUnknown23)
-
 
 class BrokenDiscarded(TestCase):
 
     def test_empty(self):
         from mutagen.id3 import TPE1, ID3JunkFrameError
-        self.assertRaises(ID3JunkFrameError, TPE1.fromData, _24, 0x00, b'')
+        self.assertRaises(ID3JunkFrameError, TPE1._fromData, _24, 0x00, b'')
 
     def test_wacky_truncated_RVA2(self):
         from mutagen.id3 import RVA2, ID3JunkFrameError
         data = b'\x01{\xf0\x10\xff\xff\x00'
-        self.assertRaises(ID3JunkFrameError, RVA2.fromData, _24, 0x00, data)
+        self.assertRaises(ID3JunkFrameError, RVA2._fromData, _24, 0x00, data)
 
     def test_bad_number_of_bits_RVA2(self):
         from mutagen.id3 import RVA2, ID3JunkFrameError
         data = b'\x00\x00\x01\xe6\xfc\x10{\xd7'
-        self.assertRaises(ID3JunkFrameError, RVA2.fromData, _24, 0x00, data)
+        self.assertRaises(ID3JunkFrameError, RVA2._fromData, _24, 0x00, data)
 
     def test_drops_truncated_frames(self):
         from mutagen.id3 import Frames
         id3 = ID3()
+        id3._header = ID3Header()
+        id3._header.version = (2, 4, 0)
         tail = b'\x00\x00\x00\x03\x00\x00' b'\x01\x02\x03'
         for head in b'RVA2 TXXX APIC'.split():
             data = head + tail
@@ -1119,19 +1107,11 @@ class BrokenDiscarded(TestCase):
         from mutagen.id3 import COMM, ID3JunkFrameError
         # 7 bytes of "UTF16" data.
         data = b'\x01\x00\x00\x00\xff\xfe\x00\xff\xfeh\x00'
-        self.assertRaises(ID3JunkFrameError, COMM.fromData, _24, 0x00, data)
+        self.assertRaises(ID3JunkFrameError, COMM._fromData, _24, 0x00, data)
 
 
 class BrokenButParsed(TestCase):
 
-    def test_missing_encoding(self):
-        from mutagen.id3 import TIT2
-        tag = TIT2.fromData(_23, 0x00, b'a test')
-        self.assertEquals(0, tag.encoding)
-        self.assertEquals('a test', tag)
-        self.assertEquals(['a test'], tag)
-        self.assertEquals(['a test'], tag.text)
-
     def test_zerolength_framedata(self):
         from mutagen.id3 import Frames
         id3 = ID3()
@@ -1143,23 +1123,23 @@ class BrokenButParsed(TestCase):
 
     def test_lengthone_utf16(self):
         from mutagen.id3 import TPE1
-        tpe1 = TPE1.fromData(_24, 0, b'\x01\x00')
+        tpe1 = TPE1._fromData(_24, 0, b'\x01\x00')
         self.assertEquals(u'', tpe1)
-        tpe1 = TPE1.fromData(_24, 0, b'\x01\x00\x00\x00\x00')
+        tpe1 = TPE1._fromData(_24, 0, b'\x01\x00\x00\x00\x00')
         self.assertEquals([u'', u''], tpe1)
 
     def test_utf16_wrongnullterm(self):
         # issue 169
         from mutagen.id3 import TPE1
-        tpe1 = TPE1.fromData(
+        tpe1 = TPE1._fromData(
             _24, 0, b'\x01\xff\xfeH\x00e\x00l\x00l\x00o\x00\x00')
         self.assertEquals(tpe1, [u'Hello'])
 
-    def test_fake_zlib_pedantic(self):
-        from mutagen.id3 import TPE1, Frame, ID3BadCompressedData
-        id3 = ID3()
-        id3.PEDANTIC = True
-        self.assertRaises(ID3BadCompressedData, TPE1.fromData, id3,
+    def test_fake_zlib(self):
+        from mutagen.id3 import TPE1, Frame
+        header = ID3Header()
+        header.version = (2, 4, 0)
+        self.assertRaises(ID3JunkFrameError, TPE1._fromData, header,
                           Frame.FLAG24_COMPRESS, b'\x03abcdefg')
 
     def test_zlib_bpi(self):
@@ -1171,16 +1151,9 @@ class BrokenButParsed(TestCase):
         self.failIf(
             max(datalen_size) >= b'\x80'[0], "data is not syncsafe: %r" % data)
 
-    def test_fake_zlib_nopedantic(self):
-        from mutagen.id3 import TPE1, Frame
-        id3 = ID3()
-        id3.PEDANTIC = False
-        tpe1 = TPE1.fromData(id3, Frame.FLAG24_COMPRESS, b'\x03abcdefg')
-        self.assertEquals(u'abcdefg', tpe1)
-
     def test_ql_0_12_missing_uncompressed_size(self):
         from mutagen.id3 import TPE1
-        tag = TPE1.fromData(
+        tag = TPE1._fromData(
             _24, 0x08,
             b'x\x9cc\xfc\xff\xaf\x84!\x83!\x93'
             b'\xa1\x98A\x01J&2\xe83\x940\xa4\x02\xd9%\x0c\x00\x87\xc6\x07#'
@@ -1190,7 +1163,7 @@ class BrokenButParsed(TestCase):
 
     def test_zlib_latin1_missing_datalen(self):
         from mutagen.id3 import TPE1
-        tag = TPE1.fromData(
+        tag = TPE1._fromData(
             _24, 0x8,
             b'\x00\x00\x00\x0f'
             b'x\x9cc(\xc9\xc8,V\x00\xa2D\xfd\x92\xd4\xe2\x12\x00&\x7f\x05%'
@@ -1201,11 +1174,15 @@ class BrokenButParsed(TestCase):
     def test_detect_23_ints_in_24_frames(self):
         from mutagen.id3 import Frames
         head = b'TIT1\x00\x00\x01\x00\x00\x00\x00'
-        tail = b'TPE1\x00\x00\x00\x04\x00\x00Yay!'
+        tail = b'TPE1\x00\x00\x00\x05\x00\x00\x00Yay!'
+
+        id3 = ID3()
+        id3._header = ID3Header()
+        id3._header.version = (2, 4, 0)
 
         tagsgood = list(
-            _24._ID3__read_frames(head + b'a' * 127 + tail, Frames))
-        tagsbad = list(_24._ID3__read_frames(head + b'a' * 255 + tail, Frames))
+            id3._ID3__read_frames(head + b'a' * 127 + tail, Frames))
+        tagsbad = list(id3._ID3__read_frames(head + b'a' * 255 + tail, Frames))
         self.assertEquals(2, len(tagsgood))
         self.assertEquals(2, len(tagsbad))
         self.assertEquals('a' * 127, tagsgood[0])
@@ -1213,8 +1190,8 @@ class BrokenButParsed(TestCase):
         self.assertEquals('Yay!', tagsgood[1])
         self.assertEquals('Yay!', tagsbad[1])
 
-        tagsgood = list(_24._ID3__read_frames(head + b'a' * 127, Frames))
-        tagsbad = list(_24._ID3__read_frames(head + b'a' * 255, Frames))
+        tagsgood = list(id3._ID3__read_frames(head + b'a' * 127, Frames))
+        tagsbad = list(id3._ID3__read_frames(head + b'a' * 255, Frames))
         self.assertEquals(1, len(tagsgood))
         self.assertEquals(1, len(tagsbad))
         self.assertEquals('a' * 127, tagsgood[0])
@@ -1222,8 +1199,8 @@ class BrokenButParsed(TestCase):
 
 
 class OddWrites(TestCase):
-    silence = join('tests', 'data', 'silence-44-s.mp3')
-    newsilence = join('tests', 'data', 'silence-written.mp3')
+    silence = join(DATA_DIR, 'silence-44-s.mp3')
+    newsilence = join(DATA_DIR, 'silence-written.mp3')
 
     def setUp(self):
         shutil.copy(self.silence, self.newsilence)
@@ -1254,8 +1231,8 @@ class OddWrites(TestCase):
 
 
 class WriteRoundtrip(TestCase):
-    silence = join('tests', 'data', 'silence-44-s.mp3')
-    newsilence = join('tests', 'data', 'silence-written.mp3')
+    silence = join(DATA_DIR, 'silence-44-s.mp3')
+    newsilence = join(DATA_DIR, 'silence-written.mp3')
 
     def setUp(self):
         shutil.copy(self.silence, self.newsilence)
@@ -1378,8 +1355,8 @@ class WriteRoundtrip(TestCase):
 
 
 class WriteForEyeD3(TestCase):
-    silence = join('tests', 'data', 'silence-44-s.mp3')
-    newsilence = join('tests', 'data', 'silence-written.mp3')
+    silence = join(DATA_DIR, 'silence-44-s.mp3')
+    newsilence = join(DATA_DIR, 'silence-written.mp3')
 
     def setUp(self):
         shutil.copy(self.silence, self.newsilence)
@@ -1426,7 +1403,7 @@ class WriteForEyeD3(TestCase):
 
 class BadTYER(TestCase):
 
-    filename = join('tests', 'data', 'bad-TYER-frame.mp3')
+    filename = join(DATA_DIR, 'bad-TYER-frame.mp3')
 
     def setUp(self):
         self.audio = ID3(self.filename)
@@ -1443,8 +1420,8 @@ class BadTYER(TestCase):
 
 class BadPOPM(TestCase):
 
-    filename = join('tests', 'data', 'bad-POPM-frame.mp3')
-    newfilename = join('tests', 'data', 'bad-POPM-frame-written.mp3')
+    filename = join(DATA_DIR, 'bad-POPM-frame.mp3')
+    newfilename = join(DATA_DIR, 'bad-POPM-frame-written.mp3')
 
     def setUp(self):
         shutil.copy(self.filename, self.newfilename)
@@ -1594,10 +1571,9 @@ class UpdateTo23(TestCase):
 
 class WriteTo23(TestCase):
 
-    SILENCE = os.path.join("tests", "data", "silence-44-s.mp3")
+    SILENCE = os.path.join(DATA_DIR, "silence-44-s.mp3")
 
     def setUp(self):
-        from tempfile import mkstemp
         fd, self.filename = mkstemp(suffix='.mp3')
         os.close(fd)
         shutil.copy(self.SILENCE, self.filename)
@@ -1674,7 +1650,6 @@ class WriteTo23(TestCase):
 class Read22FrameNamesin23(TestCase):
 
     def test_PIC_in_23(self):
-        from tempfile import mkstemp
         fd, filename = mkstemp(suffix='.mp3')
         os.close(fd)
 
@@ -1698,10 +1673,9 @@ class Read22FrameNamesin23(TestCase):
 
 
 class ID3V1_vs_APEv2(TestCase):
-    SILENCE = os.path.join("tests", "data", "silence-44-s.mp3")
+    SILENCE = os.path.join(DATA_DIR, "silence-44-s.mp3")
 
     def setUp(self):
-        from tempfile import mkstemp
         fd, self.filename = mkstemp(suffix='.mp3')
         os.close(fd)
         shutil.copy(self.SILENCE, self.filename)
@@ -1749,27 +1723,57 @@ class TID3Misc(TestCase):
         self.assertEqual(id3.Encoding.UTF8, 3)
         self.assertEqual(id3.ID3v1SaveOptions.UPDATE, 1)
 
+    def test_determine_bpi(self):
+        determine_bpi = id3._determine_bpi
+        # default to BitPaddedInt
+        self.assertTrue(determine_bpi("", {}) is BitPaddedInt)
 
-add(TID3Misc)
-add(ID3V1_vs_APEv2)
-add(Read22FrameNamesin23)
-add(ID3Loading)
-add(ID3GetSetDel)
-add(ID3Tags)
-add(ID3v1Tags)
-add(BrokenDiscarded)
-add(BrokenButParsed)
-add(WriteRoundtrip)
-add(OddWrites)
-add(BadTYER)
-add(BadPOPM)
-add(Issue69_BadV1Year)
-add(UpdateTo23)
-add(WriteTo23)
+        def get_frame_data(name, size, bpi=True):
+            data = name
+            if bpi:
+                data += BitPaddedInt.to_str(size)
+            else:
+                data += BitPaddedInt.to_str(size, bits=8)
+            data += b"\x00\x00" + b"\x01" * size
+            return data
+
+        data = get_frame_data(b"TPE2", 1000, True)
+        self.assertTrue(determine_bpi(data, Frames) is BitPaddedInt)
+        self.assertTrue(
+            determine_bpi(data + b"\x00" * 1000, Frames) is BitPaddedInt)
+
+        data = get_frame_data(b"TPE2", 1000, False)
+        self.assertTrue(determine_bpi(data, Frames) is int)
+        self.assertTrue(determine_bpi(data + b"\x00" * 1000, Frames) is int)
+
+        # in this case it helps that we know the frame name
+        d = get_frame_data(b"TPE2", 1000) + get_frame_data(b"TPE2", 10) + \
+                b"\x01" * 875
+        self.assertTrue(determine_bpi(d, Frames) is BitPaddedInt)
+
+
+class TID3Corrupt(TestCase):
+
+    def setUp(self):
+        fd, self.filename = mkstemp(suffix='.mp3')
+        os.close(fd)
+        orig = os.path.join(DATA_DIR, "silence-44-s.mp3")
+        shutil.copy(orig, self.filename)
+
+    def tearDown(self):
+        os.remove(self.filename)
+
+    def test_header_too_small(self):
+        with open(self.filename, "r+b") as h:
+            h.truncate(5)
+        self.assertRaises(id3.error, ID3, self.filename)
+
+    def test_tag_too_small(self):
+        with open(self.filename, "r+b") as h:
+            h.truncate(50)
+        self.assertRaises(id3.error, ID3, self.filename)
 
 try:
     import eyeD3
 except ImportError:
-    pass
-else:
-    add(WriteForEyeD3)
+    del WriteForEyeD3
