@@ -1,16 +1,19 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
-from os.path import devnull
 
 from cStringIO import StringIO
 from tempfile import mkstemp
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR
 
 import warnings
 warnings.simplefilter("ignore", DeprecationWarning)
 from mutagen.m4a import (M4A, Atom, Atoms, M4ATags, M4AInfo, delete, M4ACover,
                          M4AMetadataError)
 
+from tests.test_mp4 import have_faad, call_faad
+
 
 class TAtom(TestCase):
 
@@ -41,11 +44,10 @@ class TAtom(TestCase):
         fileobj = StringIO("\x00\x00\x00\x00atom")
         Atom(fileobj)
         self.failUnlessEqual(fileobj.tell(), 8)
-add(TAtom)
 
 
 class TAtoms(TestCase):
-    filename = os.path.join("tests", "data", "has-tags.m4a")
+    filename = os.path.join(DATA_DIR, "has-tags.m4a")
 
     def setUp(self):
         self.atoms = Atoms(open(self.filename, "rb"))
@@ -66,7 +68,6 @@ class TAtoms(TestCase):
 
     def test_repr(self):
         repr(self.atoms)
-add(TAtoms)
 
 
 class TM4AInfo(TestCase):
@@ -87,7 +88,6 @@ class TM4AInfo(TestCase):
         atoms = Atoms(fileobj)
         info = M4AInfo(atoms, fileobj)
         self.failUnlessEqual(info.length, 8)
-add(TM4AInfo)
 
 
 class TM4ATags(TestCase):
@@ -136,23 +136,25 @@ class TM4ATags(TestCase):
         covr = Atom.render("covr", data)
         self.failUnlessRaises(M4AMetadataError, self.wrap_ilst, covr)
 
-add(TM4ATags)
-
 
 class TM4A(TestCase):
+
     def setUp(self):
         fd, self.filename = mkstemp(suffix='m4a')
         os.close(fd)
         shutil.copy(self.original, self.filename)
         self.audio = M4A(self.filename)
 
+    def tearDown(self):
+        os.unlink(self.filename)
+
+
+class TM4AMixin(object):
+
     def faad(self):
         if not have_faad:
             return
-        value = os.system(
-            "faad %s -o %s > %s 2> %s" % (
-                self.filename, devnull, devnull, devnull))
-        self.failIf(value and value != NOTFOUND)
+        self.assertEqual(call_faad("-w", self.filename), 0)
 
     def test_bitrate(self):
         self.failUnlessEqual(self.audio.info.bitrate, 2914)
@@ -242,12 +244,9 @@ class TM4A(TestCase):
     def test_mime(self):
         self.failUnless("audio/mp4" in self.audio.mime)
 
-    def tearDown(self):
-        os.unlink(self.filename)
 
-
-class TM4AHasTags(TM4A):
-    original = os.path.join("tests", "data", "has-tags.m4a")
+class TM4AHasTags(TM4A, TM4AMixin):
+    original = os.path.join(DATA_DIR, "has-tags.m4a")
 
     def test_save_simple(self):
         self.audio.save()
@@ -269,22 +268,11 @@ class TM4AHasTags(TM4A):
 
     def test_not_my_file(self):
         self.failUnlessRaises(
-            IOError, M4A, os.path.join("tests", "data", "empty.ogg"))
-
-add(TM4AHasTags)
+            IOError, M4A, os.path.join(DATA_DIR, "empty.ogg"))
 
 
-class TM4ANoTags(TM4A):
-    original = os.path.join("tests", "data", "no-tags.m4a")
+class TM4ANoTags(TM4A, TM4AMixin):
+    original = os.path.join(DATA_DIR, "no-tags.m4a")
 
     def test_no_tags(self):
         self.failUnless(self.audio.tags is None)
-
-add(TM4ANoTags)
-
-NOTFOUND = os.system("tools/notarealprogram 2> %s" % devnull)
-
-have_faad = True
-if os.system("faad 2> %s > %s" % (devnull, devnull)) == NOTFOUND:
-    have_faad = False
-    print "WARNING: Skipping FAAD reference tests."
