@@ -1,17 +1,19 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
 import struct
+import subprocess
 
 from mutagen._compat import cBytesIO, PY3, text_type
 from tempfile import mkstemp
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR
 from mutagen.mp4 import (MP4, Atom, Atoms, MP4Tags, MP4Info, delete, MP4Cover,
                          MP4MetadataError, MP4FreeForm, error, AtomDataType,
                          MP4MetadataValueError, AtomError)
 from mutagen.mp4._util import parse_full_atom
 from mutagen.mp4._as_entry import AudioSampleEntry, ASEntryError
 from mutagen._util import cdata
-from os import devnull
 
 
 class TAtom(TestCase):
@@ -86,11 +88,9 @@ class TAtom(TestCase):
         self.assertFalse(ok)
         self.assertEqual(data, payload)
 
-add(TAtom)
-
 
 class TAtoms(TestCase):
-    filename = os.path.join("tests", "data", "has-tags.m4a")
+    filename = os.path.join(DATA_DIR, "has-tags.m4a")
 
     def setUp(self):
         self.atoms = Atoms(open(self.filename, "rb"))
@@ -120,7 +120,6 @@ class TAtoms(TestCase):
 
     def test_repr(self):
         repr(self.atoms)
-add(TAtoms)
 
 
 class TMP4Info(TestCase):
@@ -157,7 +156,6 @@ class TMP4Info(TestCase):
         atoms = Atoms(fileobj)
         info = MP4Info(atoms, fileobj)
         self.failUnlessEqual(info.length, 8)
-add(TMP4Info)
 
 
 class TMP4Tags(TestCase):
@@ -318,7 +316,7 @@ class TMP4Tags(TestCase):
         self.assertFalse(tags)
 
         # save it into an existing mp4
-        original = os.path.join("tests", "data", "has-tags.m4a")
+        original = os.path.join(DATA_DIR, "has-tags.m4a")
         fd, filename = mkstemp(suffix='.mp4')
         os.close(fd)
         shutil.copy(original, filename)
@@ -429,22 +427,25 @@ class TMP4Tags(TestCase):
         v = self.wrap_ilst(data)[key][0]
         self.failUnlessEqual(v.dataformat, AtomDataType.UTF8)
 
-add(TMP4Tags)
-
 
 class TMP4(TestCase):
+
     def setUp(self):
         fd, self.filename = mkstemp(suffix='.m4a')
         os.close(fd)
         shutil.copy(self.original, self.filename)
         self.audio = MP4(self.filename)
 
+    def tearDown(self):
+        os.unlink(self.filename)
+
+
+class TMP4Mixin(object):
+
     def faad(self):
         if not have_faad:
             return
-        value = os.system("faad %s -o %s > %s 2> %s" % (self.filename, devnull,
-                                                        devnull, devnull))
-        self.failIf(value and value != NOTFOUND)
+        self.assertEqual(call_faad("-w", self.filename), 0)
 
     def test_score(self):
         fileobj = open(self.filename, "rb")
@@ -730,11 +731,8 @@ class TMP4(TestCase):
     def test_mime(self):
         self.failUnless("audio/mp4" in self.audio.mime)
 
-    def tearDown(self):
-        os.unlink(self.filename)
 
-
-class TMP4HasTags(TMP4):
+class TMP4HasTagsMixin(TMP4Mixin):
     def test_save_simple(self):
         self.audio.save()
         self.faad()
@@ -762,11 +760,11 @@ class TMP4HasTags(TMP4):
     def test_not_my_file(self):
         # should raise something like "Not a MP4 file"
         self.failUnlessRaisesRegexp(
-            error, "MP4", MP4, os.path.join("tests", "data", "empty.ogg"))
+            error, "MP4", MP4, os.path.join(DATA_DIR, "empty.ogg"))
 
 
-class TMP4Datatypes(TMP4HasTags):
-    original = os.path.join("tests", "data", "has-tags.m4a")
+class TMP4Datatypes(TMP4, TMP4HasTagsMixin):
+    original = os.path.join(DATA_DIR, "has-tags.m4a")
 
     def test_has_freeform(self):
         key = "----:com.apple.iTunes:iTunNORM"
@@ -782,12 +780,10 @@ class TMP4Datatypes(TMP4HasTags):
         self.failUnlessEqual(covr[0].imageformat, MP4Cover.FORMAT_PNG)
         self.failUnlessEqual(covr[1].imageformat, MP4Cover.FORMAT_JPEG)
 
-add(TMP4Datatypes)
 
-
-class TMP4CovrWithName(TMP4):
+class TMP4CovrWithName(TMP4, TMP4Mixin):
     # http://bugs.musicbrainz.org/ticket/5894
-    original = os.path.join("tests", "data", "covr-with-name.m4a")
+    original = os.path.join(DATA_DIR, "covr-with-name.m4a")
 
     def test_has_covr(self):
         self.failUnless('covr' in self.audio.tags)
@@ -796,11 +792,9 @@ class TMP4CovrWithName(TMP4):
         self.failUnlessEqual(covr[0].imageformat, MP4Cover.FORMAT_PNG)
         self.failUnlessEqual(covr[1].imageformat, MP4Cover.FORMAT_JPEG)
 
-add(TMP4CovrWithName)
 
-
-class TMP4HasTags64Bit(TMP4HasTags):
-    original = os.path.join("tests", "data", "truncated-64bit.mp4")
+class TMP4HasTags64Bit(TMP4, TMP4HasTagsMixin):
+    original = os.path.join(DATA_DIR, "truncated-64bit.mp4")
 
     def test_has_covr(self):
         pass
@@ -815,11 +809,9 @@ class TMP4HasTags64Bit(TMP4HasTags):
         # This is only half a file, so FAAD segfaults. Can't test. :(
         pass
 
-add(TMP4HasTags64Bit)
 
-
-class TMP4NoTagsM4A(TMP4):
-    original = os.path.join("tests", "data", "no-tags.m4a")
+class TMP4NoTagsM4A(TMP4, TMP4Mixin):
+    original = os.path.join(DATA_DIR, "no-tags.m4a")
 
     def test_no_tags(self):
         self.failUnless(self.audio.tags is None)
@@ -828,11 +820,9 @@ class TMP4NoTagsM4A(TMP4):
         self.audio.add_tags()
         self.failUnlessRaises(error, self.audio.add_tags)
 
-add(TMP4NoTagsM4A)
 
-
-class TMP4NoTags3G2(TMP4):
-    original = os.path.join("tests", "data", "no-tags.3g2")
+class TMP4NoTags3G2(TMP4, TMP4Mixin):
+    original = os.path.join(DATA_DIR, "no-tags.3g2")
 
     def test_no_tags(self):
         self.failUnless(self.audio.tags is None)
@@ -846,11 +836,9 @@ class TMP4NoTags3G2(TMP4):
     def test_length(self):
         self.failUnlessAlmostEqual(15, self.audio.info.length, 1)
 
-add(TMP4NoTags3G2)
-
 
 class TMP4UpdateParents64Bit(TestCase):
-    original = os.path.join("tests", "data", "64bit.mp4")
+    original = os.path.join(DATA_DIR, "64bit.mp4")
 
     def setUp(self):
         fd, self.filename = mkstemp(suffix='.mp4')
@@ -875,11 +863,9 @@ class TMP4UpdateParents64Bit(TestCase):
     def tearDown(self):
         os.unlink(self.filename)
 
-add(TMP4UpdateParents64Bit)
-
 
 class TMP4ALAC(TestCase):
-    original = os.path.join("tests", "data", "alac.m4a")
+    original = os.path.join(DATA_DIR, "alac.m4a")
 
     def setUp(self):
         self.audio = MP4(self.original)
@@ -902,8 +888,6 @@ class TMP4ALAC(TestCase):
     def test_kind(self):
         self.assertEqual(self.audio.info.codec, u'alac')
 
-add(TMP4ALAC)
-
 
 class TMP4Misc(TestCase):
 
@@ -913,7 +897,16 @@ class TMP4Misc(TestCase):
 
         self.assertRaises(ValueError, parse_full_atom, b"\x00\x00\x00")
 
-add(TMP4Misc)
+    def test_sort_items(self):
+        items = [
+            ("\xa9nam", ["foo"]),
+            ("gnre", ["fo"]),
+            ("----", ["123"]),
+            ("----", ["1234"]),
+        ]
+
+        sorted_items = sorted(items, key=MP4Tags._key_sort)
+        self.assertEqual(sorted_items, items)
 
 
 class TMP4AudioSampleEntry(TestCase):
@@ -1066,12 +1059,16 @@ class TMP4AudioSampleEntry(TestCase):
         atom = Atom(fileobj)
         self.assertRaises(ASEntryError, AudioSampleEntry, atom, fileobj)
 
-add(TMP4AudioSampleEntry)
 
-
-NOTFOUND = os.system("tools/notarealprogram 2> %s" % devnull)
+def call_faad(*args):
+    with open(os.devnull, 'wb') as null:
+        return subprocess.call(
+            ["faad"] + list(args),
+            stdout=null, stderr=subprocess.STDOUT)
 
 have_faad = True
-if os.system("faad 2> %s > %s" % (devnull, devnull)) == NOTFOUND:
+try:
+    call_faad()
+except OSError:
     have_faad = False
     print("WARNING: Skipping FAAD reference tests.")
